from typing import ClassVar as _ClassVar, Optional as _Optional

from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message

DESCRIPTOR: _descriptor.FileDescriptor

class StartEngineRequest(_message.Message):
    __slots__ = ()
    def __init__(self) -> None: ...

class StartEngineResponse(_message.Message):
    __slots__ = ()
    def __init__(self) -> None: ...

class StopEngineRequest(_message.Message):
    __slots__ = ()
    def __init__(self) -> None: ...

class StopEngineResponse(_message.Message):
    __slots__ = ()
    def __init__(self) -> None: ...

class ResetEngineRequest(_message.Message):
    __slots__ = ()
    def __init__(self) -> None: ...

class ResetEngineResponse(_message.Message):
    __slots__ = ()
    def __init__(self) -> None: ...

class IncreaseTimeRequest(_message.Message):
    __slots__ = ("milliseconds",)
    MILLISECONDS_FIELD_NUMBER: _ClassVar[int]
    milliseconds: int
    def __init__(self, milliseconds: _Optional[int] = ...) -> None: ...

class IncreaseTimeResponse(_message.Message):
    __slots__ = ()
    def __init__(self) -> None: ...

class WaitForIdleStateRequest(_message.Message):
    __slots__ = ("timeout",)
    TIMEOUT_FIELD_NUMBER: _ClassVar[int]
    timeout: int
    def __init__(self, timeout: _Optional[int] = ...) -> None: ...

class WaitForIdleStateResponse(_message.Message):
    __slots__ = ()
    def __init__(self) -> None: ...

class WaitForBusyStateRequest(_message.Message):
    __slots__ = ("timeout",)
    TIMEOUT_FIELD_NUMBER: _ClassVar[int]
    timeout: int
    def __init__(self, timeout: _Optional[int] = ...) -> None: ...

class WaitForBusyStateResponse(_message.Message):
    __slots__ = ()
    def __init__(self) -> None: ...

class GetRecordsRequest(_message.Message):
    __slots__ = ()
    def __init__(self) -> None: ...

class RecordResponse(_message.Message):
    __slots__ = ("recordJson",)
    RECORDJSON_FIELD_NUMBER: _ClassVar[int]
    recordJson: str
    def __init__(self, recordJson: _Optional[str] = ...) -> None: ...



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc

from pytest_zeebe.engine.engine_control_grpc import \
    pytest_zeebe_engine_control_pb2 as pytest__zeebe_dot_engine_dot_engine__control__grpc_dot_pytest__zeebe__engine__control__pb2


class EngineControlStub(object):
    """Missing associated documentation comment in .proto file."""

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.StartEngine = channel.unary_unary(
                '/engine_protocol.EngineControl/StartEngine',
                request_serializer=pytest__zeebe_dot_engine_dot_engine__control__grpc_dot_pytest__zeebe__engine__control__pb2.StartEngineRequest.SerializeToString,
                response_deserializer=pytest__zeebe_dot_engine_dot_engine__control__grpc_dot_pytest__zeebe__engine__control__pb2.StartEngineResponse.FromString,
                )
        self.StopEngine = channel.unary_unary(
                '/engine_protocol.EngineControl/StopEngine',
                request_serializer=pytest__zeebe_dot_engine_dot_engine__control__grpc_dot_pytest__zeebe__engine__control__pb2.StopEngineRequest.SerializeToString,
                response_deserializer=pytest__zeebe_dot_engine_dot_engine__control__grpc_dot_pytest__zeebe__engine__control__pb2.StopEngineResponse.FromString,
                )
        self.ResetEngine = channel.unary_unary(
                '/engine_protocol.EngineControl/ResetEngine',
                request_serializer=pytest__zeebe_dot_engine_dot_engine__control__grpc_dot_pytest__zeebe__engine__control__pb2.ResetEngineRequest.SerializeToString,
                response_deserializer=pytest__zeebe_dot_engine_dot_engine__control__grpc_dot_pytest__zeebe__engine__control__pb2.ResetEngineResponse.FromString,
                )
        self.IncreaseTime = channel.unary_unary(
                '/engine_protocol.EngineControl/IncreaseTime',
                request_serializer=pytest__zeebe_dot_engine_dot_engine__control__grpc_dot_pytest__zeebe__engine__control__pb2.IncreaseTimeRequest.SerializeToString,
                response_deserializer=pytest__zeebe_dot_engine_dot_engine__control__grpc_dot_pytest__zeebe__engine__control__pb2.IncreaseTimeResponse.FromString,
                )
        self.WaitForIdleState = channel.unary_unary(
                '/engine_protocol.EngineControl/WaitForIdleState',
                request_serializer=pytest__zeebe_dot_engine_dot_engine__control__grpc_dot_pytest__zeebe__engine__control__pb2.WaitForIdleStateRequest.SerializeToString,
                response_deserializer=pytest__zeebe_dot_engine_dot_engine__control__grpc_dot_pytest__zeebe__engine__control__pb2.WaitForIdleStateResponse.FromString,
                )
        self.WaitForBusyState = channel.unary_unary(
                '/engine_protocol.EngineControl/WaitForBusyState',
                request_serializer=pytest__zeebe_dot_engine_dot_engine__control__grpc_dot_pytest__zeebe__engine__control__pb2.WaitForBusyStateRequest.SerializeToString,
                response_deserializer=pytest__zeebe_dot_engine_dot_engine__control__grpc_dot_pytest__zeebe__engine__control__pb2.WaitForBusyStateResponse.FromString,
                )
        self.GetRecords = channel.unary_stream(
                '/engine_protocol.EngineControl/GetRecords',
                request_serializer=pytest__zeebe_dot_engine_dot_engine__control__grpc_dot_pytest__zeebe__engine__control__pb2.GetRecordsRequest.SerializeToString,
                response_deserializer=pytest__zeebe_dot_engine_dot_engine__control__grpc_dot_pytest__zeebe__engine__control__pb2.RecordResponse.FromString,
                )


class EngineControlServicer(object):
    """Missing associated documentation comment in .proto file."""

    def StartEngine(self, request, context):
        """
        Starts the in-memory engine.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def StopEngine(self, request, context):
        """
        Stops the in-memory engine.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ResetEngine(self, request, context):
        """
        Resets the in-memory engine. This is done by recreating the engine entirely.
        The engine should be reset before executing the next test. This prevents
        the tests from influencing each other.
        Using this while running tests concurrently will still cause issues. This is
        because the first test will be using the engine, whilst the second test will
        reset it. Therefore, the data from the first test will get lost.

        After resetting the engine still needs to be started.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def IncreaseTime(self, request, context):
        """
        Increases the engine time by a specific duration. Increasing the time could
        be useful for when triggering timers with a date somewhere in the future.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def WaitForIdleState(self, request, context):
        """
        Waits for the engine to have reached an idle state. Idle state is a state in
        which the process engine makes no progress and is waiting for new commands
        or events to trigger.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def WaitForBusyState(self, request, context):
        """
        Waits a given timeout for the engine to have reached a busy state.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetRecords(self, request, context):
        """
        Get all records from the in-memory engine. These records will be returned
        in a JSON format. Client-side these should be mapped to Records.

        For an easy way to serialize this JSON back to a Record please refer to:
        https://github.com/camunda/zeebe/tree/main/protocol-jackson
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_EngineControlServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'StartEngine': grpc.unary_unary_rpc_method_handler(
                    servicer.StartEngine,
                    request_deserializer=pytest__zeebe_dot_engine_dot_engine__control__grpc_dot_pytest__zeebe__engine__control__pb2.StartEngineRequest.FromString,
                    response_serializer=pytest__zeebe_dot_engine_dot_engine__control__grpc_dot_pytest__zeebe__engine__control__pb2.StartEngineResponse.SerializeToString,
            ),
            'StopEngine': grpc.unary_unary_rpc_method_handler(
                    servicer.StopEngine,
                    request_deserializer=pytest__zeebe_dot_engine_dot_engine__control__grpc_dot_pytest__zeebe__engine__control__pb2.StopEngineRequest.FromString,
                    response_serializer=pytest__zeebe_dot_engine_dot_engine__control__grpc_dot_pytest__zeebe__engine__control__pb2.StopEngineResponse.SerializeToString,
            ),
            'ResetEngine': grpc.unary_unary_rpc_method_handler(
                    servicer.ResetEngine,
                    request_deserializer=pytest__zeebe_dot_engine_dot_engine__control__grpc_dot_pytest__zeebe__engine__control__pb2.ResetEngineRequest.FromString,
                    response_serializer=pytest__zeebe_dot_engine_dot_engine__control__grpc_dot_pytest__zeebe__engine__control__pb2.ResetEngineResponse.SerializeToString,
            ),
            'IncreaseTime': grpc.unary_unary_rpc_method_handler(
                    servicer.IncreaseTime,
                    request_deserializer=pytest__zeebe_dot_engine_dot_engine__control__grpc_dot_pytest__zeebe__engine__control__pb2.IncreaseTimeRequest.FromString,
                    response_serializer=pytest__zeebe_dot_engine_dot_engine__control__grpc_dot_pytest__zeebe__engine__control__pb2.IncreaseTimeResponse.SerializeToString,
            ),
            'WaitForIdleState': grpc.unary_unary_rpc_method_handler(
                    servicer.WaitForIdleState,
                    request_deserializer=pytest__zeebe_dot_engine_dot_engine__control__grpc_dot_pytest__zeebe__engine__control__pb2.WaitForIdleStateRequest.FromString,
                    response_serializer=pytest__zeebe_dot_engine_dot_engine__control__grpc_dot_pytest__zeebe__engine__control__pb2.WaitForIdleStateResponse.SerializeToString,
            ),
            'WaitForBusyState': grpc.unary_unary_rpc_method_handler(
                    servicer.WaitForBusyState,
                    request_deserializer=pytest__zeebe_dot_engine_dot_engine__control__grpc_dot_pytest__zeebe__engine__control__pb2.WaitForBusyStateRequest.FromString,
                    response_serializer=pytest__zeebe_dot_engine_dot_engine__control__grpc_dot_pytest__zeebe__engine__control__pb2.WaitForBusyStateResponse.SerializeToString,
            ),
            'GetRecords': grpc.unary_stream_rpc_method_handler(
                    servicer.GetRecords,
                    request_deserializer=pytest__zeebe_dot_engine_dot_engine__control__grpc_dot_pytest__zeebe__engine__control__pb2.GetRecordsRequest.FromString,
                    response_serializer=pytest__zeebe_dot_engine_dot_engine__control__grpc_dot_pytest__zeebe__engine__control__pb2.RecordResponse.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'engine_protocol.EngineControl', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))


 # This class is part of an EXPERIMENTAL API.
class EngineControl(object):
    """Missing associated documentation comment in .proto file."""

    @staticmethod
    def StartEngine(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/engine_protocol.EngineControl/StartEngine',
            pytest__zeebe_dot_engine_dot_engine__control__grpc_dot_pytest__zeebe__engine__control__pb2.StartEngineRequest.SerializeToString,
            pytest__zeebe_dot_engine_dot_engine__control__grpc_dot_pytest__zeebe__engine__control__pb2.StartEngineResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def StopEngine(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/engine_protocol.EngineControl/StopEngine',
            pytest__zeebe_dot_engine_dot_engine__control__grpc_dot_pytest__zeebe__engine__control__pb2.StopEngineRequest.SerializeToString,
            pytest__zeebe_dot_engine_dot_engine__control__grpc_dot_pytest__zeebe__engine__control__pb2.StopEngineResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def ResetEngine(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/engine_protocol.EngineControl/ResetEngine',
            pytest__zeebe_dot_engine_dot_engine__control__grpc_dot_pytest__zeebe__engine__control__pb2.ResetEngineRequest.SerializeToString,
            pytest__zeebe_dot_engine_dot_engine__control__grpc_dot_pytest__zeebe__engine__control__pb2.ResetEngineResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def IncreaseTime(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/engine_protocol.EngineControl/IncreaseTime',
            pytest__zeebe_dot_engine_dot_engine__control__grpc_dot_pytest__zeebe__engine__control__pb2.IncreaseTimeRequest.SerializeToString,
            pytest__zeebe_dot_engine_dot_engine__control__grpc_dot_pytest__zeebe__engine__control__pb2.IncreaseTimeResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def WaitForIdleState(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/engine_protocol.EngineControl/WaitForIdleState',
            pytest__zeebe_dot_engine_dot_engine__control__grpc_dot_pytest__zeebe__engine__control__pb2.WaitForIdleStateRequest.SerializeToString,
            pytest__zeebe_dot_engine_dot_engine__control__grpc_dot_pytest__zeebe__engine__control__pb2.WaitForIdleStateResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def WaitForBusyState(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/engine_protocol.EngineControl/WaitForBusyState',
            pytest__zeebe_dot_engine_dot_engine__control__grpc_dot_pytest__zeebe__engine__control__pb2.WaitForBusyStateRequest.SerializeToString,
            pytest__zeebe_dot_engine_dot_engine__control__grpc_dot_pytest__zeebe__engine__control__pb2.WaitForBusyStateResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetRecords(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_stream(request, target, '/engine_protocol.EngineControl/GetRecords',
            pytest__zeebe_dot_engine_dot_engine__control__grpc_dot_pytest__zeebe__engine__control__pb2.GetRecordsRequest.SerializeToString,
            pytest__zeebe_dot_engine_dot_engine__control__grpc_dot_pytest__zeebe__engine__control__pb2.RecordResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)



import json
import logging
import os
from typing import Tuple

import grpc

from pytest_zeebe.client.client_grpc.pytest_zeebe_gateway_pb2 import CreateProcessInstanceRequest, \
    CreateProcessInstanceWithResultRequest, CancelProcessInstanceRequest, DeployProcessResponse, DeployResourceRequest, \
    Resource, PublishMessageResponse, PublishMessageRequest, \
    BroadcastSignalResponse, BroadcastSignalRequest, EvaluateDecisionRequest, EvaluateDecisionResponse, \
    ActivateJobsRequest, ActivatedJob, CompleteJobRequest, CompleteJobResponse, \
    ThrowErrorResponse, ThrowErrorRequest, FailJobResponse, FailJobRequest, CreateProcessInstanceResponse
from pytest_zeebe.client.client_grpc.pytest_zeebe_gateway_pb2_grpc import GatewayStub
from pytest_zeebe.engine.zeebe_test_engine import ZeebeTestEngine

logger = logging.getLogger(__name__)


class ZeebeTestClient:

    def __init__(self, engine: ZeebeTestEngine) -> None:
        self.engine = engine
        self._channel = grpc.insecure_channel(f"{engine.host}:{engine.engine_port}")
        self._gateway_stub = GatewayStub(self._channel)

    def stop(self):
        try:
            self._channel.close()
        except Exception as exception:
            logger.exception("Failed to close channel, %s exception was raised", type(exception).__name__)

    def create_process_instance(
            self,
            bpmn_process_id: str,
            version: int = -1,
            variables: dict | None = None
    ) -> CreateProcessInstanceResponse:
        return self._gateway_stub.CreateProcessInstance(
            CreateProcessInstanceRequest(
                bpmnProcessId=bpmn_process_id,
                version=version,
                variables=json.dumps(variables)
            )
        )

    def create_process_instance_with_result(
            self,
            bpmn_process_id: str,
            version: int = -1,
            variables: dict | None = None,
            timeout: int = 0,
            variables_to_fetch: list | None = None
    ) -> Tuple[int, dict]:
        response = self._gateway_stub.CreateProcessInstanceWithResult(
            CreateProcessInstanceWithResultRequest(
                request=CreateProcessInstanceRequest(
                    bpmnProcessId=bpmn_process_id,
                    version=version,
                    variables=json.dumps(variables or {})
                ),
                requestTimeout=timeout,
                fetchVariables=variables_to_fetch or [],
            )
        )
        return response.processInstanceKey, json.loads(response.variables)

    def cancel_process_instance(self, process_instance_key: int) -> None:
        self._gateway_stub.CancelProcessInstance(
            CancelProcessInstanceRequest(
                processInstanceKey=process_instance_key
            )
        )

    def deploy_process(self, *process_file_path: str) -> DeployProcessResponse:
        return self._gateway_stub.DeployResource(
            DeployResourceRequest(
                resources=[result for result in map(_create_resource, process_file_path)]
            )
        )

    def publish_message(
            self,
            name: str,
            correlation_key: str,
            time_to_live_in_milliseconds: int = 60000,
            variables: dict | None = None,
            message_id: str | None = None,
    ) -> PublishMessageResponse:
        return self._gateway_stub.PublishMessage(
            PublishMessageRequest(
                name=name,
                correlationKey=correlation_key,
                messageId=message_id,
                timeToLive=time_to_live_in_milliseconds,
                variables=json.dumps(variables or {}),
            )
        )

    def broadcast_signal(self, name: str, variables: dict | None = None) -> BroadcastSignalResponse:
        return self._gateway_stub.BroadcastSignal(
            BroadcastSignalRequest(
                signalName=name,
                variables=json.dumps(variables or {}),
            )
        )

    def evaluate_decision(
            self,
            decision_key: int,
            decision_id: str,
            variables: dict | None = None
    ) -> EvaluateDecisionResponse:
        return self._gateway_stub.EvaluateDecision(
            EvaluateDecisionRequest(
                decisionKey=decision_key,
                decisionId=decision_id,
                variables=json.dumps(variables or {}),
            )
        )

    def complete_task(self, task_type: str, variables: dict | None = None):
        jobs = self._activate_jobs(task_type)
        if not jobs:
            raise Exception(f"No jobs found for task type {task_type}")
        for job in jobs:
            self._complete_job(job.key, variables)
        self.engine.wait_for_idle_state()

    def complete_user_task(self, element_id: str, variables: dict | None = None):
        user_tasks = self._activate_jobs(
            'io.camunda.zeebe:userTask',
            max_jobs_to_activate=100
        )
        for user_task in user_tasks:
            if user_task.elementId == element_id:
                self._complete_job(user_task.key, variables)
            else:
                self._fail_job(user_task.key, retries=max(user_task.retries, 1))
        self.engine.wait_for_idle_state()

    def _activate_jobs(
            self,
            task_type: str,
            worker: str = "zeebe_test_worker",
            timeout: int = 60000,
            max_jobs_to_activate: int = 1,
            variables_to_fetch: list[str] | None = None,
            request_timeout: int = 0,
    ) -> list[ActivatedJob]:
        for response in self._gateway_stub.ActivateJobs(
                ActivateJobsRequest(
                    type=task_type,
                    worker=worker,
                    timeout=timeout,
                    maxJobsToActivate=max_jobs_to_activate,
                    fetchVariable=variables_to_fetch,
                    requestTimeout=request_timeout,
                )
        ):
            return response.jobs

    def _complete_job(self, job_key: int, variables: dict | None = None) -> CompleteJobResponse:
        return self._gateway_stub.CompleteJob(
            CompleteJobRequest(
                jobKey=job_key,
                variables=json.dumps(variables or {})
            )
        )

    def _fail_job(self, job_key: int, retries: int, message: str | None = None) -> FailJobResponse:
        return self._gateway_stub.FailJob(
            FailJobRequest(
                jobKey=job_key,
                retries=retries,
                errorMessage=message
            )
        )

    def _throw_error(self, job_key: int, error_code: str, message: str = "") -> ThrowErrorResponse:
        return self._gateway_stub.ThrowError(
            ThrowErrorRequest(
                jobKey=job_key,
                errorMessage=message,
                errorCode=error_code
            )
        )


def _create_resource(process_file_path: str) -> Resource:
    with open(process_file_path, "rb") as file:
        return Resource(name=os.path.basename(process_file_path), content=file.read())



import logging
import os

import pytest
from testcontainers.core.container import DockerContainer
from testcontainers.core.waiting_utils import wait_for_logs

from pytest_zeebe.assertion import local
from pytest_zeebe.client.zeebe_test_client import ZeebeTestClient
from pytest_zeebe.engine.zeebe_test_engine import ZeebeTestEngine

logger = logging.getLogger(__name__)

DEFAULT_ZEEBE_TEST_IMAGE = "camunda/zeebe-process-test-engine"
DEFAULT_ZEEBE_TEST_IMAGE_TAG = "latest"

AGENT_PORT = 26501
ENGINE_PORT = 26500

LOG_READY_PREDICATE = "ZeebeProcessTestEngine container has started"


@pytest.fixture(scope="module")
def _container():
    image = os.environ.get("ZEEBE_TEST_IMAGE", DEFAULT_ZEEBE_TEST_IMAGE)
    tag = os.environ.get("ZEEBE_TEST_IMAGE_TAG", DEFAULT_ZEEBE_TEST_IMAGE_TAG)
    container = DockerContainer(f"{image}:{tag}")
    container.with_exposed_ports(AGENT_PORT, ENGINE_PORT)
    container.start()

    wait_for_logs(container, LOG_READY_PREDICATE)
    logger.info("Zeebe container ready")

    yield container

    logger.info("Shutting down zeebe container")
    container.stop()


@pytest.fixture(scope="module")
def _engine(_container):
    engine = ZeebeTestEngine(
        _container.get_container_host_ip(),
        str(_container.get_exposed_port(26501)),
        str(_container.get_exposed_port(26500))
    )
    local.zeebe_test_engine = engine
    return engine


@pytest.fixture
def zeebe_test_engine(_engine):
    logger.info(f"Starting zeebe engine at {_engine.host}:{_engine.engine_port}")
    _engine.start()

    yield _engine

    logger.info("Resetting zeebe engine")
    _engine.reset()


@pytest.fixture
def zeebe_test_client(zeebe_test_engine):
    logger.info("Creating zeebe client")
    client = ZeebeTestClient(zeebe_test_engine)

    yield client

    client.stop()



from typing import ClassVar as _ClassVar, Iterable as _Iterable, Mapping as _Mapping, Optional as _Optional, \
    Union as _Union

from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf.internal import containers as _containers
from google.protobuf.internal import enum_type_wrapper as _enum_type_wrapper

DESCRIPTOR: _descriptor.FileDescriptor

class ActivateJobsRequest(_message.Message):
    __slots__ = ("type", "worker", "timeout", "maxJobsToActivate", "fetchVariable", "requestTimeout")
    TYPE_FIELD_NUMBER: _ClassVar[int]
    WORKER_FIELD_NUMBER: _ClassVar[int]
    TIMEOUT_FIELD_NUMBER: _ClassVar[int]
    MAXJOBSTOACTIVATE_FIELD_NUMBER: _ClassVar[int]
    FETCHVARIABLE_FIELD_NUMBER: _ClassVar[int]
    REQUESTTIMEOUT_FIELD_NUMBER: _ClassVar[int]
    type: str
    worker: str
    timeout: int
    maxJobsToActivate: int
    fetchVariable: _containers.RepeatedScalarFieldContainer[str]
    requestTimeout: int
    def __init__(self, type: _Optional[str] = ..., worker: _Optional[str] = ..., timeout: _Optional[int] = ..., maxJobsToActivate: _Optional[int] = ..., fetchVariable: _Optional[_Iterable[str]] = ..., requestTimeout: _Optional[int] = ...) -> None: ...

class ActivateJobsResponse(_message.Message):
    __slots__ = ("jobs",)
    JOBS_FIELD_NUMBER: _ClassVar[int]
    jobs: _containers.RepeatedCompositeFieldContainer[ActivatedJob]
    def __init__(self, jobs: _Optional[_Iterable[_Union[ActivatedJob, _Mapping]]] = ...) -> None: ...

class ActivatedJob(_message.Message):
    __slots__ = ("key", "type", "processInstanceKey", "bpmnProcessId", "processDefinitionVersion", "processDefinitionKey", "elementId", "elementInstanceKey", "customHeaders", "worker", "retries", "deadline", "variables")
    KEY_FIELD_NUMBER: _ClassVar[int]
    TYPE_FIELD_NUMBER: _ClassVar[int]
    PROCESSINSTANCEKEY_FIELD_NUMBER: _ClassVar[int]
    BPMNPROCESSID_FIELD_NUMBER: _ClassVar[int]
    PROCESSDEFINITIONVERSION_FIELD_NUMBER: _ClassVar[int]
    PROCESSDEFINITIONKEY_FIELD_NUMBER: _ClassVar[int]
    ELEMENTID_FIELD_NUMBER: _ClassVar[int]
    ELEMENTINSTANCEKEY_FIELD_NUMBER: _ClassVar[int]
    CUSTOMHEADERS_FIELD_NUMBER: _ClassVar[int]
    WORKER_FIELD_NUMBER: _ClassVar[int]
    RETRIES_FIELD_NUMBER: _ClassVar[int]
    DEADLINE_FIELD_NUMBER: _ClassVar[int]
    VARIABLES_FIELD_NUMBER: _ClassVar[int]
    key: int
    type: str
    processInstanceKey: int
    bpmnProcessId: str
    processDefinitionVersion: int
    processDefinitionKey: int
    elementId: str
    elementInstanceKey: int
    customHeaders: str
    worker: str
    retries: int
    deadline: int
    variables: str
    def __init__(self, key: _Optional[int] = ..., type: _Optional[str] = ..., processInstanceKey: _Optional[int] = ..., bpmnProcessId: _Optional[str] = ..., processDefinitionVersion: _Optional[int] = ..., processDefinitionKey: _Optional[int] = ..., elementId: _Optional[str] = ..., elementInstanceKey: _Optional[int] = ..., customHeaders: _Optional[str] = ..., worker: _Optional[str] = ..., retries: _Optional[int] = ..., deadline: _Optional[int] = ..., variables: _Optional[str] = ...) -> None: ...

class CancelProcessInstanceRequest(_message.Message):
    __slots__ = ("processInstanceKey",)
    PROCESSINSTANCEKEY_FIELD_NUMBER: _ClassVar[int]
    processInstanceKey: int
    def __init__(self, processInstanceKey: _Optional[int] = ...) -> None: ...

class CancelProcessInstanceResponse(_message.Message):
    __slots__ = ()
    def __init__(self) -> None: ...

class CompleteJobRequest(_message.Message):
    __slots__ = ("jobKey", "variables")
    JOBKEY_FIELD_NUMBER: _ClassVar[int]
    VARIABLES_FIELD_NUMBER: _ClassVar[int]
    jobKey: int
    variables: str
    def __init__(self, jobKey: _Optional[int] = ..., variables: _Optional[str] = ...) -> None: ...

class CompleteJobResponse(_message.Message):
    __slots__ = ()
    def __init__(self) -> None: ...

class CreateProcessInstanceRequest(_message.Message):
    __slots__ = ("processDefinitionKey", "bpmnProcessId", "version", "variables", "startInstructions")
    PROCESSDEFINITIONKEY_FIELD_NUMBER: _ClassVar[int]
    BPMNPROCESSID_FIELD_NUMBER: _ClassVar[int]
    VERSION_FIELD_NUMBER: _ClassVar[int]
    VARIABLES_FIELD_NUMBER: _ClassVar[int]
    STARTINSTRUCTIONS_FIELD_NUMBER: _ClassVar[int]
    processDefinitionKey: int
    bpmnProcessId: str
    version: int
    variables: str
    startInstructions: _containers.RepeatedCompositeFieldContainer[ProcessInstanceCreationStartInstruction]
    def __init__(self, processDefinitionKey: _Optional[int] = ..., bpmnProcessId: _Optional[str] = ..., version: _Optional[int] = ..., variables: _Optional[str] = ..., startInstructions: _Optional[_Iterable[_Union[ProcessInstanceCreationStartInstruction, _Mapping]]] = ...) -> None: ...

class ProcessInstanceCreationStartInstruction(_message.Message):
    __slots__ = ("elementId",)
    ELEMENTID_FIELD_NUMBER: _ClassVar[int]
    elementId: str
    def __init__(self, elementId: _Optional[str] = ...) -> None: ...

class CreateProcessInstanceResponse(_message.Message):
    __slots__ = ("processDefinitionKey", "bpmnProcessId", "version", "processInstanceKey")
    PROCESSDEFINITIONKEY_FIELD_NUMBER: _ClassVar[int]
    BPMNPROCESSID_FIELD_NUMBER: _ClassVar[int]
    VERSION_FIELD_NUMBER: _ClassVar[int]
    PROCESSINSTANCEKEY_FIELD_NUMBER: _ClassVar[int]
    processDefinitionKey: int
    bpmnProcessId: str
    version: int
    processInstanceKey: int
    def __init__(self, processDefinitionKey: _Optional[int] = ..., bpmnProcessId: _Optional[str] = ..., version: _Optional[int] = ..., processInstanceKey: _Optional[int] = ...) -> None: ...

class CreateProcessInstanceWithResultRequest(_message.Message):
    __slots__ = ("request", "requestTimeout", "fetchVariables")
    REQUEST_FIELD_NUMBER: _ClassVar[int]
    REQUESTTIMEOUT_FIELD_NUMBER: _ClassVar[int]
    FETCHVARIABLES_FIELD_NUMBER: _ClassVar[int]
    request: CreateProcessInstanceRequest
    requestTimeout: int
    fetchVariables: _containers.RepeatedScalarFieldContainer[str]
    def __init__(self, request: _Optional[_Union[CreateProcessInstanceRequest, _Mapping]] = ..., requestTimeout: _Optional[int] = ..., fetchVariables: _Optional[_Iterable[str]] = ...) -> None: ...

class CreateProcessInstanceWithResultResponse(_message.Message):
    __slots__ = ("processDefinitionKey", "bpmnProcessId", "version", "processInstanceKey", "variables")
    PROCESSDEFINITIONKEY_FIELD_NUMBER: _ClassVar[int]
    BPMNPROCESSID_FIELD_NUMBER: _ClassVar[int]
    VERSION_FIELD_NUMBER: _ClassVar[int]
    PROCESSINSTANCEKEY_FIELD_NUMBER: _ClassVar[int]
    VARIABLES_FIELD_NUMBER: _ClassVar[int]
    processDefinitionKey: int
    bpmnProcessId: str
    version: int
    processInstanceKey: int
    variables: str
    def __init__(self, processDefinitionKey: _Optional[int] = ..., bpmnProcessId: _Optional[str] = ..., version: _Optional[int] = ..., processInstanceKey: _Optional[int] = ..., variables: _Optional[str] = ...) -> None: ...

class EvaluateDecisionRequest(_message.Message):
    __slots__ = ("decisionKey", "decisionId", "variables")
    DECISIONKEY_FIELD_NUMBER: _ClassVar[int]
    DECISIONID_FIELD_NUMBER: _ClassVar[int]
    VARIABLES_FIELD_NUMBER: _ClassVar[int]
    decisionKey: int
    decisionId: str
    variables: str
    def __init__(self, decisionKey: _Optional[int] = ..., decisionId: _Optional[str] = ..., variables: _Optional[str] = ...) -> None: ...

class EvaluateDecisionResponse(_message.Message):
    __slots__ = ("decisionKey", "decisionId", "decisionName", "decisionVersion", "decisionRequirementsId", "decisionRequirementsKey", "decisionOutput", "evaluatedDecisions", "failedDecisionId", "failureMessage")
    DECISIONKEY_FIELD_NUMBER: _ClassVar[int]
    DECISIONID_FIELD_NUMBER: _ClassVar[int]
    DECISIONNAME_FIELD_NUMBER: _ClassVar[int]
    DECISIONVERSION_FIELD_NUMBER: _ClassVar[int]
    DECISIONREQUIREMENTSID_FIELD_NUMBER: _ClassVar[int]
    DECISIONREQUIREMENTSKEY_FIELD_NUMBER: _ClassVar[int]
    DECISIONOUTPUT_FIELD_NUMBER: _ClassVar[int]
    EVALUATEDDECISIONS_FIELD_NUMBER: _ClassVar[int]
    FAILEDDECISIONID_FIELD_NUMBER: _ClassVar[int]
    FAILUREMESSAGE_FIELD_NUMBER: _ClassVar[int]
    decisionKey: int
    decisionId: str
    decisionName: str
    decisionVersion: int
    decisionRequirementsId: str
    decisionRequirementsKey: int
    decisionOutput: str
    evaluatedDecisions: _containers.RepeatedCompositeFieldContainer[EvaluatedDecision]
    failedDecisionId: str
    failureMessage: str
    def __init__(self, decisionKey: _Optional[int] = ..., decisionId: _Optional[str] = ..., decisionName: _Optional[str] = ..., decisionVersion: _Optional[int] = ..., decisionRequirementsId: _Optional[str] = ..., decisionRequirementsKey: _Optional[int] = ..., decisionOutput: _Optional[str] = ..., evaluatedDecisions: _Optional[_Iterable[_Union[EvaluatedDecision, _Mapping]]] = ..., failedDecisionId: _Optional[str] = ..., failureMessage: _Optional[str] = ...) -> None: ...

class EvaluatedDecision(_message.Message):
    __slots__ = ("decisionKey", "decisionId", "decisionName", "decisionVersion", "decisionType", "decisionOutput", "matchedRules", "evaluatedInputs")
    DECISIONKEY_FIELD_NUMBER: _ClassVar[int]
    DECISIONID_FIELD_NUMBER: _ClassVar[int]
    DECISIONNAME_FIELD_NUMBER: _ClassVar[int]
    DECISIONVERSION_FIELD_NUMBER: _ClassVar[int]
    DECISIONTYPE_FIELD_NUMBER: _ClassVar[int]
    DECISIONOUTPUT_FIELD_NUMBER: _ClassVar[int]
    MATCHEDRULES_FIELD_NUMBER: _ClassVar[int]
    EVALUATEDINPUTS_FIELD_NUMBER: _ClassVar[int]
    decisionKey: int
    decisionId: str
    decisionName: str
    decisionVersion: int
    decisionType: str
    decisionOutput: str
    matchedRules: _containers.RepeatedCompositeFieldContainer[MatchedDecisionRule]
    evaluatedInputs: _containers.RepeatedCompositeFieldContainer[EvaluatedDecisionInput]
    def __init__(self, decisionKey: _Optional[int] = ..., decisionId: _Optional[str] = ..., decisionName: _Optional[str] = ..., decisionVersion: _Optional[int] = ..., decisionType: _Optional[str] = ..., decisionOutput: _Optional[str] = ..., matchedRules: _Optional[_Iterable[_Union[MatchedDecisionRule, _Mapping]]] = ..., evaluatedInputs: _Optional[_Iterable[_Union[EvaluatedDecisionInput, _Mapping]]] = ...) -> None: ...

class EvaluatedDecisionInput(_message.Message):
    __slots__ = ("inputId", "inputName", "inputValue")
    INPUTID_FIELD_NUMBER: _ClassVar[int]
    INPUTNAME_FIELD_NUMBER: _ClassVar[int]
    INPUTVALUE_FIELD_NUMBER: _ClassVar[int]
    inputId: str
    inputName: str
    inputValue: str
    def __init__(self, inputId: _Optional[str] = ..., inputName: _Optional[str] = ..., inputValue: _Optional[str] = ...) -> None: ...

class EvaluatedDecisionOutput(_message.Message):
    __slots__ = ("outputId", "outputName", "outputValue")
    OUTPUTID_FIELD_NUMBER: _ClassVar[int]
    OUTPUTNAME_FIELD_NUMBER: _ClassVar[int]
    OUTPUTVALUE_FIELD_NUMBER: _ClassVar[int]
    outputId: str
    outputName: str
    outputValue: str
    def __init__(self, outputId: _Optional[str] = ..., outputName: _Optional[str] = ..., outputValue: _Optional[str] = ...) -> None: ...

class MatchedDecisionRule(_message.Message):
    __slots__ = ("ruleId", "ruleIndex", "evaluatedOutputs")
    RULEID_FIELD_NUMBER: _ClassVar[int]
    RULEINDEX_FIELD_NUMBER: _ClassVar[int]
    EVALUATEDOUTPUTS_FIELD_NUMBER: _ClassVar[int]
    ruleId: str
    ruleIndex: int
    evaluatedOutputs: _containers.RepeatedCompositeFieldContainer[EvaluatedDecisionOutput]
    def __init__(self, ruleId: _Optional[str] = ..., ruleIndex: _Optional[int] = ..., evaluatedOutputs: _Optional[_Iterable[_Union[EvaluatedDecisionOutput, _Mapping]]] = ...) -> None: ...

class DeployProcessRequest(_message.Message):
    __slots__ = ("processes",)
    PROCESSES_FIELD_NUMBER: _ClassVar[int]
    processes: _containers.RepeatedCompositeFieldContainer[ProcessRequestObject]
    def __init__(self, processes: _Optional[_Iterable[_Union[ProcessRequestObject, _Mapping]]] = ...) -> None: ...

class ProcessRequestObject(_message.Message):
    __slots__ = ("name", "definition")
    NAME_FIELD_NUMBER: _ClassVar[int]
    DEFINITION_FIELD_NUMBER: _ClassVar[int]
    name: str
    definition: bytes
    def __init__(self, name: _Optional[str] = ..., definition: _Optional[bytes] = ...) -> None: ...

class DeployProcessResponse(_message.Message):
    __slots__ = ("key", "processes")
    KEY_FIELD_NUMBER: _ClassVar[int]
    PROCESSES_FIELD_NUMBER: _ClassVar[int]
    key: int
    processes: _containers.RepeatedCompositeFieldContainer[ProcessMetadata]
    def __init__(self, key: _Optional[int] = ..., processes: _Optional[_Iterable[_Union[ProcessMetadata, _Mapping]]] = ...) -> None: ...

class DeployResourceRequest(_message.Message):
    __slots__ = ("resources",)
    RESOURCES_FIELD_NUMBER: _ClassVar[int]
    resources: _containers.RepeatedCompositeFieldContainer[Resource]
    def __init__(self, resources: _Optional[_Iterable[_Union[Resource, _Mapping]]] = ...) -> None: ...

class Resource(_message.Message):
    __slots__ = ("name", "content")
    NAME_FIELD_NUMBER: _ClassVar[int]
    CONTENT_FIELD_NUMBER: _ClassVar[int]
    name: str
    content: bytes
    def __init__(self, name: _Optional[str] = ..., content: _Optional[bytes] = ...) -> None: ...

class DeployResourceResponse(_message.Message):
    __slots__ = ("key", "deployments")
    KEY_FIELD_NUMBER: _ClassVar[int]
    DEPLOYMENTS_FIELD_NUMBER: _ClassVar[int]
    key: int
    deployments: _containers.RepeatedCompositeFieldContainer[Deployment]
    def __init__(self, key: _Optional[int] = ..., deployments: _Optional[_Iterable[_Union[Deployment, _Mapping]]] = ...) -> None: ...

class Deployment(_message.Message):
    __slots__ = ("process", "decision", "decisionRequirements")
    PROCESS_FIELD_NUMBER: _ClassVar[int]
    DECISION_FIELD_NUMBER: _ClassVar[int]
    DECISIONREQUIREMENTS_FIELD_NUMBER: _ClassVar[int]
    process: ProcessMetadata
    decision: DecisionMetadata
    decisionRequirements: DecisionRequirementsMetadata
    def __init__(self, process: _Optional[_Union[ProcessMetadata, _Mapping]] = ..., decision: _Optional[_Union[DecisionMetadata, _Mapping]] = ..., decisionRequirements: _Optional[_Union[DecisionRequirementsMetadata, _Mapping]] = ...) -> None: ...

class ProcessMetadata(_message.Message):
    __slots__ = ("bpmnProcessId", "version", "processDefinitionKey", "resourceName")
    BPMNPROCESSID_FIELD_NUMBER: _ClassVar[int]
    VERSION_FIELD_NUMBER: _ClassVar[int]
    PROCESSDEFINITIONKEY_FIELD_NUMBER: _ClassVar[int]
    RESOURCENAME_FIELD_NUMBER: _ClassVar[int]
    bpmnProcessId: str
    version: int
    processDefinitionKey: int
    resourceName: str
    def __init__(self, bpmnProcessId: _Optional[str] = ..., version: _Optional[int] = ..., processDefinitionKey: _Optional[int] = ..., resourceName: _Optional[str] = ...) -> None: ...

class DecisionMetadata(_message.Message):
    __slots__ = ("dmnDecisionId", "dmnDecisionName", "version", "decisionKey", "dmnDecisionRequirementsId", "decisionRequirementsKey")
    DMNDECISIONID_FIELD_NUMBER: _ClassVar[int]
    DMNDECISIONNAME_FIELD_NUMBER: _ClassVar[int]
    VERSION_FIELD_NUMBER: _ClassVar[int]
    DECISIONKEY_FIELD_NUMBER: _ClassVar[int]
    DMNDECISIONREQUIREMENTSID_FIELD_NUMBER: _ClassVar[int]
    DECISIONREQUIREMENTSKEY_FIELD_NUMBER: _ClassVar[int]
    dmnDecisionId: str
    dmnDecisionName: str
    version: int
    decisionKey: int
    dmnDecisionRequirementsId: str
    decisionRequirementsKey: int
    def __init__(self, dmnDecisionId: _Optional[str] = ..., dmnDecisionName: _Optional[str] = ..., version: _Optional[int] = ..., decisionKey: _Optional[int] = ..., dmnDecisionRequirementsId: _Optional[str] = ..., decisionRequirementsKey: _Optional[int] = ...) -> None: ...

class DecisionRequirementsMetadata(_message.Message):
    __slots__ = ("dmnDecisionRequirementsId", "dmnDecisionRequirementsName", "version", "decisionRequirementsKey", "resourceName")
    DMNDECISIONREQUIREMENTSID_FIELD_NUMBER: _ClassVar[int]
    DMNDECISIONREQUIREMENTSNAME_FIELD_NUMBER: _ClassVar[int]
    VERSION_FIELD_NUMBER: _ClassVar[int]
    DECISIONREQUIREMENTSKEY_FIELD_NUMBER: _ClassVar[int]
    RESOURCENAME_FIELD_NUMBER: _ClassVar[int]
    dmnDecisionRequirementsId: str
    dmnDecisionRequirementsName: str
    version: int
    decisionRequirementsKey: int
    resourceName: str
    def __init__(self, dmnDecisionRequirementsId: _Optional[str] = ..., dmnDecisionRequirementsName: _Optional[str] = ..., version: _Optional[int] = ..., decisionRequirementsKey: _Optional[int] = ..., resourceName: _Optional[str] = ...) -> None: ...

class FailJobRequest(_message.Message):
    __slots__ = ("jobKey", "retries", "errorMessage", "retryBackOff", "variables")
    JOBKEY_FIELD_NUMBER: _ClassVar[int]
    RETRIES_FIELD_NUMBER: _ClassVar[int]
    ERRORMESSAGE_FIELD_NUMBER: _ClassVar[int]
    RETRYBACKOFF_FIELD_NUMBER: _ClassVar[int]
    VARIABLES_FIELD_NUMBER: _ClassVar[int]
    jobKey: int
    retries: int
    errorMessage: str
    retryBackOff: int
    variables: str
    def __init__(self, jobKey: _Optional[int] = ..., retries: _Optional[int] = ..., errorMessage: _Optional[str] = ..., retryBackOff: _Optional[int] = ..., variables: _Optional[str] = ...) -> None: ...

class FailJobResponse(_message.Message):
    __slots__ = ()
    def __init__(self) -> None: ...

class ThrowErrorRequest(_message.Message):
    __slots__ = ("jobKey", "errorCode", "errorMessage", "variables")
    JOBKEY_FIELD_NUMBER: _ClassVar[int]
    ERRORCODE_FIELD_NUMBER: _ClassVar[int]
    ERRORMESSAGE_FIELD_NUMBER: _ClassVar[int]
    VARIABLES_FIELD_NUMBER: _ClassVar[int]
    jobKey: int
    errorCode: str
    errorMessage: str
    variables: str
    def __init__(self, jobKey: _Optional[int] = ..., errorCode: _Optional[str] = ..., errorMessage: _Optional[str] = ..., variables: _Optional[str] = ...) -> None: ...

class ThrowErrorResponse(_message.Message):
    __slots__ = ()
    def __init__(self) -> None: ...

class PublishMessageRequest(_message.Message):
    __slots__ = ("name", "correlationKey", "timeToLive", "messageId", "variables")
    NAME_FIELD_NUMBER: _ClassVar[int]
    CORRELATIONKEY_FIELD_NUMBER: _ClassVar[int]
    TIMETOLIVE_FIELD_NUMBER: _ClassVar[int]
    MESSAGEID_FIELD_NUMBER: _ClassVar[int]
    VARIABLES_FIELD_NUMBER: _ClassVar[int]
    name: str
    correlationKey: str
    timeToLive: int
    messageId: str
    variables: str
    def __init__(self, name: _Optional[str] = ..., correlationKey: _Optional[str] = ..., timeToLive: _Optional[int] = ..., messageId: _Optional[str] = ..., variables: _Optional[str] = ...) -> None: ...

class PublishMessageResponse(_message.Message):
    __slots__ = ("key",)
    KEY_FIELD_NUMBER: _ClassVar[int]
    key: int
    def __init__(self, key: _Optional[int] = ...) -> None: ...

class ResolveIncidentRequest(_message.Message):
    __slots__ = ("incidentKey",)
    INCIDENTKEY_FIELD_NUMBER: _ClassVar[int]
    incidentKey: int
    def __init__(self, incidentKey: _Optional[int] = ...) -> None: ...

class ResolveIncidentResponse(_message.Message):
    __slots__ = ()
    def __init__(self) -> None: ...

class TopologyRequest(_message.Message):
    __slots__ = ()
    def __init__(self) -> None: ...

class TopologyResponse(_message.Message):
    __slots__ = ("brokers", "clusterSize", "partitionsCount", "replicationFactor", "gatewayVersion")
    BROKERS_FIELD_NUMBER: _ClassVar[int]
    CLUSTERSIZE_FIELD_NUMBER: _ClassVar[int]
    PARTITIONSCOUNT_FIELD_NUMBER: _ClassVar[int]
    REPLICATIONFACTOR_FIELD_NUMBER: _ClassVar[int]
    GATEWAYVERSION_FIELD_NUMBER: _ClassVar[int]
    brokers: _containers.RepeatedCompositeFieldContainer[BrokerInfo]
    clusterSize: int
    partitionsCount: int
    replicationFactor: int
    gatewayVersion: str
    def __init__(self, brokers: _Optional[_Iterable[_Union[BrokerInfo, _Mapping]]] = ..., clusterSize: _Optional[int] = ..., partitionsCount: _Optional[int] = ..., replicationFactor: _Optional[int] = ..., gatewayVersion: _Optional[str] = ...) -> None: ...

class BrokerInfo(_message.Message):
    __slots__ = ("nodeId", "host", "port", "partitions", "version")
    NODEID_FIELD_NUMBER: _ClassVar[int]
    HOST_FIELD_NUMBER: _ClassVar[int]
    PORT_FIELD_NUMBER: _ClassVar[int]
    PARTITIONS_FIELD_NUMBER: _ClassVar[int]
    VERSION_FIELD_NUMBER: _ClassVar[int]
    nodeId: int
    host: str
    port: int
    partitions: _containers.RepeatedCompositeFieldContainer[Partition]
    version: str
    def __init__(self, nodeId: _Optional[int] = ..., host: _Optional[str] = ..., port: _Optional[int] = ..., partitions: _Optional[_Iterable[_Union[Partition, _Mapping]]] = ..., version: _Optional[str] = ...) -> None: ...

class Partition(_message.Message):
    __slots__ = ("partitionId", "role", "health")
    class PartitionBrokerRole(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
        __slots__ = ()
        LEADER: _ClassVar[Partition.PartitionBrokerRole]
        FOLLOWER: _ClassVar[Partition.PartitionBrokerRole]
        INACTIVE: _ClassVar[Partition.PartitionBrokerRole]
    LEADER: Partition.PartitionBrokerRole
    FOLLOWER: Partition.PartitionBrokerRole
    INACTIVE: Partition.PartitionBrokerRole
    class PartitionBrokerHealth(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
        __slots__ = ()
        HEALTHY: _ClassVar[Partition.PartitionBrokerHealth]
        UNHEALTHY: _ClassVar[Partition.PartitionBrokerHealth]
        DEAD: _ClassVar[Partition.PartitionBrokerHealth]
    HEALTHY: Partition.PartitionBrokerHealth
    UNHEALTHY: Partition.PartitionBrokerHealth
    DEAD: Partition.PartitionBrokerHealth
    PARTITIONID_FIELD_NUMBER: _ClassVar[int]
    ROLE_FIELD_NUMBER: _ClassVar[int]
    HEALTH_FIELD_NUMBER: _ClassVar[int]
    partitionId: int
    role: Partition.PartitionBrokerRole
    health: Partition.PartitionBrokerHealth
    def __init__(self, partitionId: _Optional[int] = ..., role: _Optional[_Union[Partition.PartitionBrokerRole, str]] = ..., health: _Optional[_Union[Partition.PartitionBrokerHealth, str]] = ...) -> None: ...

class UpdateJobRetriesRequest(_message.Message):
    __slots__ = ("jobKey", "retries")
    JOBKEY_FIELD_NUMBER: _ClassVar[int]
    RETRIES_FIELD_NUMBER: _ClassVar[int]
    jobKey: int
    retries: int
    def __init__(self, jobKey: _Optional[int] = ..., retries: _Optional[int] = ...) -> None: ...

class UpdateJobRetriesResponse(_message.Message):
    __slots__ = ()
    def __init__(self) -> None: ...

class SetVariablesRequest(_message.Message):
    __slots__ = ("elementInstanceKey", "variables", "local")
    ELEMENTINSTANCEKEY_FIELD_NUMBER: _ClassVar[int]
    VARIABLES_FIELD_NUMBER: _ClassVar[int]
    LOCAL_FIELD_NUMBER: _ClassVar[int]
    elementInstanceKey: int
    variables: str
    local: bool
    def __init__(self, elementInstanceKey: _Optional[int] = ..., variables: _Optional[str] = ..., local: bool = ...) -> None: ...

class SetVariablesResponse(_message.Message):
    __slots__ = ("key",)
    KEY_FIELD_NUMBER: _ClassVar[int]
    key: int
    def __init__(self, key: _Optional[int] = ...) -> None: ...

class ModifyProcessInstanceRequest(_message.Message):
    __slots__ = ("processInstanceKey", "activateInstructions", "terminateInstructions")
    class ActivateInstruction(_message.Message):
        __slots__ = ("elementId", "ancestorElementInstanceKey", "variableInstructions")
        ELEMENTID_FIELD_NUMBER: _ClassVar[int]
        ANCESTORELEMENTINSTANCEKEY_FIELD_NUMBER: _ClassVar[int]
        VARIABLEINSTRUCTIONS_FIELD_NUMBER: _ClassVar[int]
        elementId: str
        ancestorElementInstanceKey: int
        variableInstructions: _containers.RepeatedCompositeFieldContainer[ModifyProcessInstanceRequest.VariableInstruction]
        def __init__(self, elementId: _Optional[str] = ..., ancestorElementInstanceKey: _Optional[int] = ..., variableInstructions: _Optional[_Iterable[_Union[ModifyProcessInstanceRequest.VariableInstruction, _Mapping]]] = ...) -> None: ...
    class VariableInstruction(_message.Message):
        __slots__ = ("variables", "scopeId")
        VARIABLES_FIELD_NUMBER: _ClassVar[int]
        SCOPEID_FIELD_NUMBER: _ClassVar[int]
        variables: str
        scopeId: str
        def __init__(self, variables: _Optional[str] = ..., scopeId: _Optional[str] = ...) -> None: ...
    class TerminateInstruction(_message.Message):
        __slots__ = ("elementInstanceKey",)
        ELEMENTINSTANCEKEY_FIELD_NUMBER: _ClassVar[int]
        elementInstanceKey: int
        def __init__(self, elementInstanceKey: _Optional[int] = ...) -> None: ...
    PROCESSINSTANCEKEY_FIELD_NUMBER: _ClassVar[int]
    ACTIVATEINSTRUCTIONS_FIELD_NUMBER: _ClassVar[int]
    TERMINATEINSTRUCTIONS_FIELD_NUMBER: _ClassVar[int]
    processInstanceKey: int
    activateInstructions: _containers.RepeatedCompositeFieldContainer[ModifyProcessInstanceRequest.ActivateInstruction]
    terminateInstructions: _containers.RepeatedCompositeFieldContainer[ModifyProcessInstanceRequest.TerminateInstruction]
    def __init__(self, processInstanceKey: _Optional[int] = ..., activateInstructions: _Optional[_Iterable[_Union[ModifyProcessInstanceRequest.ActivateInstruction, _Mapping]]] = ..., terminateInstructions: _Optional[_Iterable[_Union[ModifyProcessInstanceRequest.TerminateInstruction, _Mapping]]] = ...) -> None: ...

class ModifyProcessInstanceResponse(_message.Message):
    __slots__ = ()
    def __init__(self) -> None: ...

class DeleteResourceRequest(_message.Message):
    __slots__ = ("resourceKey",)
    RESOURCEKEY_FIELD_NUMBER: _ClassVar[int]
    resourceKey: int
    def __init__(self, resourceKey: _Optional[int] = ...) -> None: ...

class DeleteResourceResponse(_message.Message):
    __slots__ = ()
    def __init__(self) -> None: ...

class BroadcastSignalRequest(_message.Message):
    __slots__ = ("signalName", "variables")
    SIGNALNAME_FIELD_NUMBER: _ClassVar[int]
    VARIABLES_FIELD_NUMBER: _ClassVar[int]
    signalName: str
    variables: str
    def __init__(self, signalName: _Optional[str] = ..., variables: _Optional[str] = ...) -> None: ...

class BroadcastSignalResponse(_message.Message):
    __slots__ = ("key",)
    KEY_FIELD_NUMBER: _ClassVar[int]
    key: int
    def __init__(self, key: _Optional[int] = ...) -> None: ...




class ProcessMessageSubscriptionRecordFilter:
    def __init__(self, records):
        self.records = [record for record in records if record["valueType"] == "PROCESS_MESSAGE_SUBSCRIPTION"]

    def with_process_instance_key(self, process_instance_key):
        return ProcessMessageSubscriptionRecordFilter(
            [record for record in self.records if record['value']['processInstanceKey'] == process_instance_key])

    def with_message_name(self, message_name):
        return ProcessMessageSubscriptionRecordFilter(
            [record for record in self.records if record['value']['messageName'] == message_name])

    def with_correlation_key(self, correlation_key):
        return ProcessMessageSubscriptionRecordFilter(
            [record for record in self.records if record['value']['correlationKey'] == correlation_key])

    def with_rejection_type(self, rejection_type):
        return ProcessMessageSubscriptionRecordFilter(
            [record for record in self.records if record['rejectionType'] == rejection_type])

    def with_intent(self, intent):
        return ProcessMessageSubscriptionRecordFilter(
            [record for record in self.records if record['intent'] == intent])

    def with_message_key(self, message_key):
        return ProcessMessageSubscriptionRecordFilter(
            [record for record in self.records if record['value']['messageKey'] == message_key])


import json

from pytest_zeebe.filters.incident_filter import IncidentRecordFilter
from pytest_zeebe.filters.process_instance_filter import ProcessInstanceRecordFilter
from pytest_zeebe.filters.process_message_subscription_filter import ProcessMessageSubscriptionRecordFilter
from pytest_zeebe.filters.variable_filter import VariableRecordFilter


class ProcessInstanceAssert:

    def __init__(self, process_instance_key, record_stream):
        self.process_instance_key = process_instance_key
        self.record_stream = record_stream

    def is_started(self):
        is_started = any(ProcessInstanceRecordFilter(self.record_stream)
                         .with_process_instance_key(self.process_instance_key)
                         .with_rejection_type('NULL_VAL')
                         .with_intent('ELEMENT_ACTIVATED')
                         .with_bpmn_element_type('PROCESS')
                         .records)
        assert is_started, f"Process with key {self.process_instance_key} was not started."
        return self

    def is_active(self):
        is_active = not any(record['intent'] in ['ELEMENT_COMPLETED', 'ELEMENT_TERMINATED'] for record in
                            ProcessInstanceRecordFilter(self.record_stream)
                            .with_process_instance_key(self.process_instance_key)
                            .with_rejection_type('NULL_VAL')
                            .with_bpmn_element_type('PROCESS')
                            .records)
        assert is_active, f"Process with key {self.process_instance_key} is not active."
        return self

    def is_completed(self):
        completed = any(ProcessInstanceRecordFilter(self.record_stream)
                        .with_process_instance_key(self.process_instance_key)
                        .with_rejection_type('NULL_VAL')
                        .with_bpmn_element_type('PROCESS')
                        .with_intent('ELEMENT_COMPLETED')
                        .records)
        assert completed, f"Process instance {self.process_instance_key} was not completed."
        return self

    def is_not_completed(self):
        not_completed = not any(ProcessInstanceRecordFilter(self.record_stream)
                                .with_process_instance_key(self.process_instance_key)
                                .with_rejection_type('NULL_VAL')
                                .with_bpmn_element_type('PROCESS')
                                .with_intent('ELEMENT_COMPLETED')
                                .records)
        assert not_completed, f"Process with key {self.process_instance_key} was completed."
        return self

    def is_terminated(self):
        is_terminated = any(ProcessInstanceRecordFilter(self.record_stream)
                            .with_process_instance_key(self.process_instance_key)
                            .with_rejection_type('NULL_VAL')
                            .with_bpmn_element_type('PROCESS')
                            .with_intent('ELEMENT_TERMINATED')
                            .records)
        assert is_terminated, f"Process with key {self.process_instance_key} was not terminated."
        return self

    def is_not_terminated(self):
        not_terminated = not any(ProcessInstanceRecordFilter(self.record_stream)
                                 .with_process_instance_key(self.process_instance_key)
                                 .with_rejection_type('NULL_VAL')
                                 .with_bpmn_element_type('PROCESS')
                                 .with_intent('ELEMENT_TERMINATED')
                                 .records)
        assert not_terminated, f"Process with key {self.process_instance_key} was terminated."
        return self

    def has_passed_element(self, element_id, times=1):
        count = sum(1 for _ in ProcessInstanceRecordFilter(self.record_stream)
                    .with_process_instance_key(self.process_instance_key)
                    .with_rejection_type('NULL_VAL')
                    .with_element_id(element_id)
                    .with_intents('ELEMENT_COMPLETED', 'SEQUENCE_FLOW_TAKEN')
                    .records)
        assert count == times, f"Expected element with id {element_id} to be passed {times} times, but was {count}."
        return self

    def has_not_passed_element(self, element_id):
        return self.has_passed_element(element_id, times=0)

    def has_passed_elements_in_order(self, *element_ids):
        found_element_records = [record['value']['elementId'] for record in
                                 ProcessInstanceRecordFilter(self.record_stream)
                                 .with_process_instance_key(self.process_instance_key)
                                 .with_rejection_type('NULL_VAL')
                                 .with_element_id_in(*element_ids)
                                 .with_intents('ELEMENT_COMPLETED', 'SEQUENCE_FLOW_TAKEN')
                                 .records]
        assert found_element_records == list(element_ids), "Ordered elements do not match."
        return self

    def is_waiting_at_elements(self, *element_ids):
        elements_in_wait_state = self.get_elements_in_wait_state()
        assert set(element_ids).issubset(elements_in_wait_state), "Not all elements are in wait state."
        return self

    def is_not_waiting_at_elements(self, *element_ids):
        elements_in_wait_state = self.get_elements_in_wait_state()
        assert not set(element_ids).intersection(elements_in_wait_state), "Some elements are in wait state."
        return self

    def get_elements_in_wait_state(self):
        elements_in_wait_state = set()
        for record in (ProcessInstanceRecordFilter(self.record_stream)
                        .with_process_instance_key(self.process_instance_key)
                        .with_rejection_type('NULL_VAL')
                        .without_bpmn_element_type('PROCESS')
                        .records):
            if record['intent'] == 'ELEMENT_ACTIVATED':
                elements_in_wait_state.add(record['value']['elementId'])
        return elements_in_wait_state

    def is_waiting_exactly_at_elements(self, *element_ids):
        elements_in_wait_state = self.get_elements_in_wait_state()
        wrongfully_waiting_element_ids = [id for id in elements_in_wait_state if id not in element_ids]
        wrongfully_not_waiting_element_ids = [id for id in element_ids if id not in elements_in_wait_state]

        assert not wrongfully_waiting_element_ids, f"Process with key {self.process_instance_key} is waiting at element(s) with id(s) {', '.join(wrongfully_waiting_element_ids)}"
        assert not wrongfully_not_waiting_element_ids, f"Process with key {self.process_instance_key} is not waiting at element(s) with id(s) {', '.join(wrongfully_not_waiting_element_ids)}"
        return self

    def is_waiting_for_messages(self, *message_names):
        open_message_subscriptions = self.get_open_message_subscriptions()
        assert set(message_names).issubset(open_message_subscriptions), "Not all messages are in open subscriptions."
        return self

    def is_not_waiting_for_messages(self, *message_names):
        open_message_subscriptions = self.get_open_message_subscriptions()
        assert not set(message_names).intersection(open_message_subscriptions), "Some messages are in open subscriptions."
        return self

    def get_open_message_subscriptions(self):
        open_message_subscriptions = set()
        for record in (ProcessMessageSubscriptionRecordFilter(self.record_stream)
            .with_process_instance_key(self.process_instance_key)
            .with_rejection_type('NULL_VAL')
            .records):
            if record['intent'] in ['CREATING', 'CREATED']:
                open_message_subscriptions.add(record['value']['messageName'])
        return open_message_subscriptions

    def has_correlated_message_by_name(self, message_name, times):
        actual_times = sum(1 for record in ProcessMessageSubscriptionRecordFilter(self.record_stream)
                           .with_process_instance_key(self.process_instance_key)
                           .with_rejection_type('NULL_VAL')
                           .with_message_name(message_name)
                           .with_intent('CORRELATED')
                           .records)
        assert actual_times == times, f"Expected message with name '{message_name}' to be correlated {times} times, but was {actual_times} times"
        return self

    def has_correlated_message_by_correlation_key(self, correlation_key, times):
        actual_times = sum(1 for record in ProcessMessageSubscriptionRecordFilter(self.record_stream)
                           .with_process_instance_key(self.process_instance_key)
                           .with_rejection_type('NULL_VAL')
                           .with_correlation_key(correlation_key)
                           .with_intent('CORRELATED')
                           .records)
        assert actual_times == times, f"Expected message with correlation key '{correlation_key}' to be correlated {times} times, but was {actual_times} times"
        return self

    def has_variable(self, name):
        variables = self.get_process_instance_variables()
        assert name in variables, f"Variable '{name}' not found."
        return self

    def has_variable_with_value(self, name, value):
        variables = self.get_process_instance_variables()
        actual_value = json.loads(variables.get(name))
        assert actual_value == value, f"Variable '{name}' does not have the expected value."
        return self

    def get_process_instance_variables(self):
        return {record['value']['name']: record['value']['value'] for record in
                VariableRecordFilter(self.record_stream)
                .with_process_instance_key(self.process_instance_key)
                .with_rejection_type('NULL_VAL')
                .records}

    def has_any_incidents(self):
        incidents_were_raised = any(True for _ in IncidentRecordFilter(self.record_stream)
                                    .with_rejection_type('NULL_VAL')
                                    .with_intent('CREATED')
                                    .with_process_instance_key(self.process_instance_key)
                                    .records)
        assert incidents_were_raised, "No incidents were raised for this process instance"
        return self

    def has_no_incidents(self):
        incidents_were_raised = any(True for _ in IncidentRecordFilter(self.record_stream)
                                    .with_rejection_type('NULL_VAL')
                                    .with_intent('CREATED')
                                    .with_process_instance_key(self.process_instance_key)
                                    .records)
        assert not incidents_were_raised, "Incidents were raised for this process instance"
        return self

    # todo implement IncidentAssert
    # def extracting_latest_incident(self):
    #     self.has_any_incidents()
    #     incident_created_records = [record for record in IncidentRecordFilter(self.record_stream)
    #                                 .with_rejection_type('NULL_VAL')
    #                                 .with_intent('CREATED')
    #                                 .with_process_instance_key(self.process_instance_key)
    #                                 .records]
    #     latest_incident_record = incident_created_records[-1]
    #     return IncidentAssert(latest_incident_record['key'], self.record_stream)

    def extracting_latest_called_process(self):
        self.has_called_process()
        latest_called_process_record = [record for record in ProcessInstanceRecordFilter(self.record_stream)
                                        .with_parent_process_instance_key(self.process_instance_key)
                                        .records][-1]
        return ProcessInstanceAssert(latest_called_process_record['key'], self.record_stream)

    def extracting_latest_called_process_with_id(self, process_id):
        self.has_called_process_with_id(process_id)
        latest_called_process_record = [record for record in ProcessInstanceRecordFilter(self.record_stream)
                                        .with_parent_process_instance_key(self.process_instance_key)
                                        .with_bpmn_process_id(process_id)
                                        .records][-1]
        return ProcessInstanceAssert(latest_called_process_record['key'], self.record_stream)

    def has_called_process(self):
        has_called_process = any(True for _ in ProcessInstanceRecordFilter(self.record_stream)
                                 .with_parent_process_instance_key(self.process_instance_key)
                                 .records)
        assert has_called_process, "No process was called from this process"
        return self

    def has_not_called_process(self):
        called_processes = {record['value']['bpmnProcessId'] for record in ProcessInstanceRecordFilter(self.record_stream)
                            .with_parent_process_instance_key(self.process_instance_key)
                            .records}
        assert not called_processes, f"A process was called from this process, distinct called processes are: {', '.join(called_processes)}"
        return self

    def has_called_process_with_id(self, process_id):
        has_called_process = any(True for record in ProcessInstanceRecordFilter(self.record_stream)
                                 .with_parent_process_instance_key(self.process_instance_key)
                                 .with_bpmn_process_id(process_id)
                                 .records)
        assert has_called_process, f"No process with id `{process_id}` was called from this process"
        return self

    def has_not_called_process_with_id(self, process_id):
        has_called_process = any(True for record in ProcessInstanceRecordFilter(self.record_stream)
                                 .with_parent_process_instance_key(self.process_instance_key)
                                 .with_bpmn_process_id(process_id)
                                 .records)
        assert not has_called_process, f"A process with id `{process_id}` was called from this process"
        return self

    def get_called_process_records(self):
        return ProcessInstanceRecordFilter(self.record_stream).with_parent_process_instance_key(
            self.process_instance_key
        )



import json

import grpc

from pytest_zeebe.engine.engine_control_grpc import pytest_zeebe_engine_control_pb2, \
    pytest_zeebe_engine_control_pb2_grpc
from pytest_zeebe.engine.engine_control_grpc.pytest_zeebe_engine_control_pb2_grpc import EngineControlStub


class ZeebeTestEngine:

    def __init__(self, host, control_port, engine_port):
        self.host = host
        self.control_port = control_port
        self.engine_port = engine_port

    def _control_channel(self) -> grpc.Channel:
        return grpc.insecure_channel(f'{self.host}:{self.control_port}')

    def start(self):
        with self._control_channel() as channel:
            stub = _create_stub(channel)
            stub.StartEngine(pytest_zeebe_engine_control_pb2.StartEngineRequest())

    def stop(self):
        with self._control_channel() as channel:
            stub = _create_stub(channel)
            stub.StopEngine(pytest_zeebe_engine_control_pb2.StopEngineRequest())

    def reset(self):
        with self._control_channel() as channel:
            stub = _create_stub(channel)
            stub.ResetEngine(pytest_zeebe_engine_control_pb2.ResetEngineRequest())

    def wait_for_idle_state(self, timeout_seconds: int = 60):
        with self._control_channel() as channel:
            stub = _create_stub(channel)
            stub.WaitForIdleState(pytest_zeebe_engine_control_pb2.WaitForIdleStateRequest(timeout=timeout_seconds * 1000))

    def get_records(self):
        records = []
        with self._control_channel() as channel:
            stub = _create_stub(channel)
            for r in stub.GetRecords(pytest_zeebe_engine_control_pb2.GetRecordsRequest()):
                records.append(json.loads(r.recordJson))
        return records


def _create_stub(channel: grpc.Channel) -> EngineControlStub:
    return pytest_zeebe_engine_control_pb2_grpc.EngineControlStub(channel)




class VariableRecordFilter:
    def __init__(self, records):
        self.records = [record for record in records if record["valueType"] == "VARIABLE"]

    def with_process_instance_key(self, process_instance_key):
        return VariableRecordFilter(
            [record for record in self.records if record['value']['processInstanceKey'] == process_instance_key])

    def with_rejection_type(self, rejection_type):
        return VariableRecordFilter(
            [record for record in self.records if record['rejectionType'] == rejection_type])


# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: pytest_zeebe/engine/engine_control_grpc/pytest_zeebe_engine_control.proto
# Protobuf Python Version: 4.25.0
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\nIpytest_zeebe/engine/engine_control_grpc/pytest_zeebe_engine_control.proto\x12\x0f\x65ngine_protocol\"\x14\n\x12StartEngineRequest\"\x15\n\x13StartEngineResponse\"\x13\n\x11StopEngineRequest\"\x14\n\x12StopEngineResponse\"\x14\n\x12ResetEngineRequest\"\x15\n\x13ResetEngineResponse\"+\n\x13IncreaseTimeRequest\x12\x14\n\x0cmilliseconds\x18\x01 \x01(\x05\"\x16\n\x14IncreaseTimeResponse\"*\n\x17WaitForIdleStateRequest\x12\x0f\n\x07timeout\x18\x01 \x01(\x03\"\x1a\n\x18WaitForIdleStateResponse\"*\n\x17WaitForBusyStateRequest\x12\x0f\n\x07timeout\x18\x01 \x01(\x03\"\x1a\n\x18WaitForBusyStateResponse\"\x13\n\x11GetRecordsRequest\"$\n\x0eRecordResponse\x12\x12\n\nrecordJson\x18\x01 \x01(\t2\x9e\x05\n\rEngineControl\x12X\n\x0bStartEngine\x12#.engine_protocol.StartEngineRequest\x1a$.engine_protocol.StartEngineResponse\x12U\n\nStopEngine\x12\".engine_protocol.StopEngineRequest\x1a#.engine_protocol.StopEngineResponse\x12X\n\x0bResetEngine\x12#.engine_protocol.ResetEngineRequest\x1a$.engine_protocol.ResetEngineResponse\x12[\n\x0cIncreaseTime\x12$.engine_protocol.IncreaseTimeRequest\x1a%.engine_protocol.IncreaseTimeResponse\x12g\n\x10WaitForIdleState\x12(.engine_protocol.WaitForIdleStateRequest\x1a).engine_protocol.WaitForIdleStateResponse\x12g\n\x10WaitForBusyState\x12(.engine_protocol.WaitForBusyStateRequest\x1a).engine_protocol.WaitForBusyStateResponse\x12S\n\nGetRecords\x12\".engine_protocol.GetRecordsRequest\x1a\x1f.engine_protocol.RecordResponse0\x01\x42\x31\n-io.camunda.zeebe.process.test.engine.protocolP\x00\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'pytest_zeebe.engine.engine_control_grpc.pytest_zeebe_engine_control_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
  _globals['DESCRIPTOR']._options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n-io.camunda.zeebe.process.test.engine.protocolP\000'
  _globals['_STARTENGINEREQUEST']._serialized_start=94
  _globals['_STARTENGINEREQUEST']._serialized_end=114
  _globals['_STARTENGINERESPONSE']._serialized_start=116
  _globals['_STARTENGINERESPONSE']._serialized_end=137
  _globals['_STOPENGINEREQUEST']._serialized_start=139
  _globals['_STOPENGINEREQUEST']._serialized_end=158
  _globals['_STOPENGINERESPONSE']._serialized_start=160
  _globals['_STOPENGINERESPONSE']._serialized_end=180
  _globals['_RESETENGINEREQUEST']._serialized_start=182
  _globals['_RESETENGINEREQUEST']._serialized_end=202
  _globals['_RESETENGINERESPONSE']._serialized_start=204
  _globals['_RESETENGINERESPONSE']._serialized_end=225
  _globals['_INCREASETIMEREQUEST']._serialized_start=227
  _globals['_INCREASETIMEREQUEST']._serialized_end=270
  _globals['_INCREASETIMERESPONSE']._serialized_start=272
  _globals['_INCREASETIMERESPONSE']._serialized_end=294
  _globals['_WAITFORIDLESTATEREQUEST']._serialized_start=296
  _globals['_WAITFORIDLESTATEREQUEST']._serialized_end=338
  _globals['_WAITFORIDLESTATERESPONSE']._serialized_start=340
  _globals['_WAITFORIDLESTATERESPONSE']._serialized_end=366
  _globals['_WAITFORBUSYSTATEREQUEST']._serialized_start=368
  _globals['_WAITFORBUSYSTATEREQUEST']._serialized_end=410
  _globals['_WAITFORBUSYSTATERESPONSE']._serialized_start=412
  _globals['_WAITFORBUSYSTATERESPONSE']._serialized_end=438
  _globals['_GETRECORDSREQUEST']._serialized_start=440
  _globals['_GETRECORDSREQUEST']._serialized_end=459
  _globals['_RECORDRESPONSE']._serialized_start=461
  _globals['_RECORDRESPONSE']._serialized_end=497
  _globals['_ENGINECONTROL']._serialized_start=500
  _globals['_ENGINECONTROL']._serialized_end=1170
# @@protoc_insertion_point(module_scope)



import threading

from pytest_zeebe.assertions.process_instance_assert import ProcessInstanceAssert
from pytest_zeebe.client.client_grpc.pytest_zeebe_gateway_pb2 import CreateProcessInstanceResponse

local = threading.local()


def assert_that(process_instance: CreateProcessInstanceResponse):
    engine = local.zeebe_test_engine
    engine.wait_for_idle_state()
    return ProcessInstanceAssert(
        process_instance.processInstanceKey,
        engine.get_records()
    )








class IncidentRecordFilter:
    def __init__(self, records):
        self.records = [record for record in records if record["valueType"] == "INCIDENT"]

    def with_incident_key(self, incident_key):
        return IncidentRecordFilter(
            [record for record in self.records if record['key'] == incident_key])

    def with_rejection_type(self, rejection_type):
        return IncidentRecordFilter(
            [record for record in self.records if record['rejectionType'] == rejection_type])

    def with_process_instance_key(self, process_instance_key):
        return IncidentRecordFilter(
            [record for record in self.records if record['value']['processInstanceKey'] == process_instance_key])

    def with_job_key(self, job_key):
        return IncidentRecordFilter(
            [record for record in self.records if record['value']['jobKey'] == job_key])

    def with_intent(self, intent):
        return IncidentRecordFilter(
            [record for record in self.records if record['intent'] == intent])



class ProcessInstanceRecordFilter:
    def __init__(self, records):
        self.records = [record for record in records if record["valueType"] == "PROCESS_INSTANCE"]

    def with_process_instance_key(self, process_instance_key):
        return ProcessInstanceRecordFilter(
            [record for record in self.records if record['value']['processInstanceKey'] == process_instance_key])

    def with_bpmn_element_type(self, bpmn_element_type):
        return ProcessInstanceRecordFilter(
            [record for record in self.records if record['value']['bpmnElementType'] == bpmn_element_type])

    def without_bpmn_element_type(self, bpmn_element_type):
        return ProcessInstanceRecordFilter(
            [record for record in self.records if record['value']['bpmnElementType'] != bpmn_element_type])

    def with_bpmn_event_type(self, bpmn_event_type):
        return ProcessInstanceRecordFilter(
            [record for record in self.records if record['value']['bpmnEventType'] == bpmn_event_type])

    def without_bpmn_event_type(self, bpmn_event_type):
        return ProcessInstanceRecordFilter(
            [record for record in self.records if record['value']['bpmnEventType'] != bpmn_event_type])

    def with_intent(self, intent):
        return ProcessInstanceRecordFilter(
            [record for record in self.records if record['intent'] == intent])

    def with_intents(self, *intents):
        return ProcessInstanceRecordFilter(
            [record for record in self.records if record['intent'] in intents])

    def with_element_id(self, element_id):
        return ProcessInstanceRecordFilter(
            [record for record in self.records if record['value']['elementId'] == element_id])

    def with_element_id_in(self, *element_ids):
        return ProcessInstanceRecordFilter(
            [record for record in self.records if record['value']['elementId'] in element_ids])

    def with_rejection_type(self, rejection_type):
        return ProcessInstanceRecordFilter(
            [record for record in self.records if record['rejectionType'] == rejection_type])

    def with_parent_process_instance_key(self, parent_process_instance_key):
        return ProcessInstanceRecordFilter(
            [record for record in self.records if record['value']['parentProcessInstanceKey'] == parent_process_instance_key])

    def with_bpmn_process_id(self, bpmn_process_id):
        return ProcessInstanceRecordFilter(
            [record for record in self.records if record['value']['bpmnProcessId'] == bpmn_process_id])

    def with_record_type(self, record_type):
        return ProcessInstanceRecordFilter(
            [record for record in self.records if record['recordType'] == record_type])
